import { storeFile } from "../utils/ipfs";
import { getOrcaClient } from "@_koii/task-manager/extensions";
import { namespaceWrapper, TASK_ID } from "@_koii/namespace-wrapper";
import { status } from "../utils/constant";
export async function submission(roundNumber: number) : Promise<string | void> {
  /**
   * Retrieve the task proofs from your container and submit for auditing
   * Must return a string of max 512 bytes to be submitted on chain
   * The default implementation handles uploading the proofs to IPFS
   * and returning the CID
   */
  console.log(`[SUBMISSION] Starting submission process for round ${roundNumber}`);

  try {
    console.log("[SUBMISSION] Initializing Orca client...");
    const orcaClient = await getOrcaClient();
    if (!orcaClient) {
      console.error("[SUBMISSION] Failed to initialize Orca client");
      return;
    }
    console.log("[SUBMISSION] Orca client initialized successfully");
    console.log(`[SUBMISSION] Fetching task result for round ${roundNumber}...`);
    const shouldMakeSubmission = await namespaceWrapper.storeGet(`shouldMakeSubmission`);
    if (!shouldMakeSubmission || shouldMakeSubmission !== "true") {
      const taskResult = await namespaceWrapper.storeGet(`result-${roundNumber}`);
      if (!taskResult) {
        console.log("[SUBMISSION] No task result found for this round");
        return status.NO_DATA_FOR_THIS_ROUND;
      }
      return taskResult;
    }
    const submissionRoundNumber = await namespaceWrapper.storeGet(`submissionRoundNumber`);
    if (!submissionRoundNumber) {
      console.log("[SUBMISSION] No submission Round Number found for this round");
      return status.NO_DATA_FOR_THIS_ROUND;
    }
    console.log(`[SUBMISSION] Fetching submission data for round ${roundNumber}. and submission roundnumber ${submissionRoundNumber}`);
    const result = await orcaClient.podCall(`submission/${submissionRoundNumber}`);
    let submission;
    console.log("[SUBMISSION] Submission result:", result);
    console.log("[SUBMISSION] Submission result data:", result.data);
  
    if (!result || result.data === "No submission") {
      console.log("[SUBMISSION] No existing submission found");
      return status.NO_SUBMISSION_BUT_SUBMISSION_CALLED;
    } else {
      // Add extra error handling for https://koii-workspace.slack.com/archives/C0886H01JM8/p1746137232538419
      if (typeof result.data === 'object' && 'data' in result.data) {
        console.log("[SUBMISSION] Submission result data is an object with 'data' property");
        submission = result.data.data;
      } else {
        console.log("[SUBMISSION] Submission result data is not an object with 'data' property");
        submission = result.data;
      }
    }

    console.log("[SUBMISSION] Validating submission data...");
    console.log("[SUBMISSION] Submission data:", submission);
    if (submission.roundNumber !== roundNumber) {
      console.error(`[SUBMISSION] Round number mismatch. Expected: ${roundNumber}, Got: ${submission.roundNumber}`);
      throw new Error("Submission is not for the current round");
    }

    if (!submission.prUrl) {
      console.error("[SUBMISSION] Missing PR URL in submission");
      throw new Error("Submission is missing PR URL");
    }

    console.log("[SUBMISSION] Submission data validated successfully:", submission);

    console.log("[SUBMISSION] Getting submitter account...");
    const stakingKeypair = await namespaceWrapper.getSubmitterAccount();

    if (!stakingKeypair) {
      console.error("[SUBMISSION] No staking keypair found");
      throw new Error("No staking keypair found");
    }
    console.log("[SUBMISSION] Submitter account retrieved successfully");

    const stakingKey = stakingKeypair.publicKey.toBase58();
    const pubKey = await namespaceWrapper.getMainAccountPubkey();
    console.log("[SUBMISSION] Staking key:", stakingKey);
    console.log("[SUBMISSION] Public key:", pubKey);

    console.log("[SUBMISSION] Signing submission payload...");
    const signature = await namespaceWrapper.payloadSigning(
      {
        taskId: TASK_ID,
        roundNumber,
        stakingKey,
        pubKey,
        action: "audit",
        ...submission,
      },
      stakingKeypair.secretKey,
    );
    console.log("[SUBMISSION] Payload signed successfully");

    console.log("[SUBMISSION] Storing submission on IPFS...");
    const cid = await storeFile({ signature }, "submission.json");
    console.log("[SUBMISSION] Submission stored successfully. CID:", cid);
    // If done please set the shouldMakeSubmission to false
    await namespaceWrapper.storeSet(`shouldMakeSubmission`, "false");
    return cid || void 0;
  } catch (error) {
    console.error("[SUBMISSION] Error during submission process:", error);
    throw error;
  }
}